from bs4 import BeautifulSoup as bs
import requests
import re

headers = {'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.96 Safari/537.36',
			'Host':'accounts.pixiv.net',
			'Referer':'https://accounts.pixiv.net/login',
			'Content-Type':'text/html; charset=UTF-8'
		}

headers2 = {
		'User-Agent': 'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/58.0.3029.96 Safari/537.36',
		'Host':'accounts.pixiv.net',
		'Referer':'https://accounts.pixiv.net/login',
		'Content-Type':'application/x-www-form-urlencoded'
}

# get提交参数
params ={
		'lang': 'en',
		'source': 'pc',
		'view_type': 'page',
		'ref': 'wwwtop_accounts_index'
		}

# post提交参数
datas = {
		'pixiv_id': '122527455@qq.com',
		'password': 'ldhllzLLZ-520',
		'captcha': '',
		'g_reaptcha_response': '',
		'post_key': '',
		'source': 'pc',
		'ref': 'wwwtop_accounts_indes',
		'return_to': 'https://www.pixiv.net'
		# 'reaptcha_enterprise_score_token':'03AGdBq25_End4u7euuSMRoIPN8CGb9Ds1Ued6Rm9syQawaIyV2tAJVZUw8HYOnbAjWMmqDhESEMZVt_dB1tmUL2kBwMH1hp2OnabHkeL7xmfQuxpaHUqVBxeIKO9TWQjSYIb2hefXVinUqGJ0yEbqX6kRvvtF4PKoxTocosd9P7LD1_GEv8ULYuQnBMPOTZM5N0lSXsEwU_Ay7OGTwzEbf41WsnS1fS_NXYepBZRzIY3JBJaw2PlKD7FtsYbwUeC6A_Up8AFcClJjMDUEbjWPPxnHEC01NYMGy0LgZeBThp7lOQubzN7-XuyD2LdYWLlk1373Ua2d9RU2LLrhsL57ZGcTFEgMmDI9QQc_ylcnDGaG-OxYJ_-5sftI7UAjpQEV52C2Uu67UybDsIxok8uhCcn3s7z0qva3apCDw0y-jHm7BnPhlCqCqaLhw324S-auWH1nd120e2DlaiXP-bNtX_LgAtQXFONgTExB9O9MHvcwDkiebYVrnkKdrTOz4xvvh5lAU5oWxei85cmO28ZRQI1Jo6JdjTlvVZRC_SfFGpZ1HsLt3oPYeJLqx14K4BjL1bk8T6xCV3H8kjLtxu8WOTUm1M6ZLkCeEpvAdUZLGwWotc5UkMx2E-0-MjdcvhCwR5FfaF65HT4YyMreOh2wIfuuy6-lRjNYSyAFrrsx12rZoxh_mar5a8DE06dutB7HENcSCVXS5Us8avUbXAClOobPzDP8EALu1ukbUAecc-yiUA90VglAys_NQsNCVNmnPSDQ-gtF3sZ9a_lEZkcuOXzEB1kWWeZgRMPErl5nKpebRldQ105sT273kQmIeHU9-zMPUGYHqQogfG5dvRHvd1vr1Glg2pRpvnuPC8fVx2OrbI__xKewlC9qdFZNkvJrFzWXRL45rGlTpeRY0_qpLPLuz183twezw5IJ-aTMygW-PXLYkJkW_LYNwGuUa48coXsbYAbYiXu0_xhkYmfmuhf_7rBOjs6hJENB5dWQuT4yXTgWpjUMG7i_xMIXN0TL1f1AMD3bgBORIa8WeWRNA_wWc9FIpDvfmGpqAJuEoJEt6U3fNCqsPwN-plSDC3bDmMswyyK60hL1v2wgm5eGgS9dd-Vlab3aRfTixO_G_F9rxfSqUg1uITCPHxZ-GUv02pIhIWv4SJjfi-V2LwDh3M0DV5JFeN3IlgdebXR4Mj_JCVqN9DuXdZgQUUYI-52X5PBueMNARVQzvDMBToWDd_540qHFBbB-44Ydmw_4_jUHXpCNfWOMPIGrOtW9m_x1KZuvKDVRWsEWBE4uoXfR5nbzd4eoSG3MibYscFkDyAAACXooYQDifHXfiyB7sBZ3PYLe8bbvR6IApLsHklC72lYc6bv5wWqYDmz6FrZUpSQexuZVuKkNqE27LuQOTjaj23SSy5OXznYEVpUoqfDD8008fxibYvmBKwj5ft-zcrbmP-VgrNjmz38MX3WbC1CKxG96I8dHorm7'
	}

login_url = 'https://210.140.131.206/login' # 登陆的URL
post_url = 'https://210.140.131.206/api/login?lang=en' # 提交POST请求的URL
analytics_url1 = 'https://www.google-analytics.com/j/collect?v=1&_v=j87&a=2064393434&t=pageview&_s=1&dl=https%3A%2F%2Faccounts.pixiv.net%2Flogin%3Freturn_to%3Dhttps%253A%252F%252Fwww.pixiv.net%252F%26lang%3Dzh%26source%3Dpc%26view_type%3Dpage&dr=https%3A%2F%2Fwww.pixiv.net%2F&ul=zh-cn&de=UTF-8&dt=%E7%99%BB%E5%BD%95%20%7C%20pixiv&sd=24-bit&sr=1440x900&vp=1440x737&je=0&fl=32.0%20r0&_utma=235335808.867814591.1612518455.1612607565.1612610035.4&_utmz=235335808.1612518455.1.1.utmcsr%3D(direct)%7Cutmccn%3D(direct)%7Cutmcmd%3D(none)&_utmht=1612613208458&_u=AAECAEABAAAAAC~&jid=762514891&gjid=800111070&cid=867814591.1612518455&tid=UA-76252338-1&_gid=614123860.1612518483&_r=1&_slc=1&z=317754011'
analytics_url2 = 'https://www.google-analytics.com/j/collect?v=1&_v=j87&a=2064393434&t=pageview&_s=1&dl=https%3A%2F%2Faccounts.pixiv.net%2Flogin%3Freturn_to%3Dhttps%253A%252F%252Fwww.pixiv.net%252F%26lang%3Dzh%26source%3Dpc%26view_type%3Dpage&dr=https%3A%2F%2Fwww.pixiv.net%2F&ul=zh-cn&de=UTF-8&dt=%E7%99%BB%E5%BD%95%20%7C%20pixiv&sd=24-bit&sr=1440x900&vp=1440x737&je=0&fl=32.0%20r0&_utma=235335808.867814591.1612518455.1612607565.1612610035.4&_utmz=235335808.1612518455.1.1.utmcsr%3D(direct)%7Cutmccn%3D(direct)%7Cutmcmd%3D(none)&_utmht=1612613208472&_u=QAECAEABAAAAAC~&jid=1985710891&gjid=307760021&cid=867814591.1612518455&tid=UA-76252338-4&_gid=614123860.1612518483&_r=1&gtm=2wg1r0KF8TRV&z=424454930'

s = requests.Session()

# 获取登录页面
res = s.get(login_url, headers = headers, verify=False)

parsed = bs(res.text,'lxml')
inp = parsed.find_all('input')
for i in inp:
	if i.get('name')=='post_key':
		post_key = i.get('value')
		break

print(res.text)

# 获取post_key
# pattern = re.compile(r'name="post_key" value="(.*?)">')
# r = pattern.findall(res.text)
# datas['post_key'] = r[0]
datas['post_key'] = post_key
# 模拟登录
result = s.post(post_url, headers = headers2, data = datas, verify=False)

# 打印出json信息
print(result.json())



r"""@2022 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool using website <https://saucenao.com/> to search & match anime pics
"""

import requests as rq
from bs4 import BeautifulSoup as bs

test_image = 'https://i.pximg.net/c/240x480/img-master/img/2022/02/24/00/00/10/96477280_p0_master1200.jpg'
search_url = 'https://saucenao.com/search.php?url='
user_url = 'https://saucenao.com/user.php'
host = 'https://saucenao.com'

header = {
	'origin': host,
	'referer': user_url,
	'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.51 Safari/537.36'
}

global login_res
login_res = None

s = rq.Session()

def login():
	up = s.get(user_url,headers = header)
	token = bs(up.text,'lxml').find('input',{'name':'token'}).attrs['value']
	data = {
		'username': 'hardrelice',
		'password': 'h0a9r2d3relice',
		'token': token
	}
	header['token'] = token
	return s.post(url=f'{user_url}?page=login', headers=header, data=data)

def search_deprecate(url):
	global login_res
	if not login_res:
		login_res = login()
	res = s.get(search_url+url, headers=header)
	if res.status_code != 200:
		login()
		res = s.get(search_url+url, headers=header)
	page = bs(res.text,'lxml')
	results = page.find_all(class_ = 'resulttable')
	if len(results)==0:
		return -1
	package = []
	for result in results:
		similarity_str = result.find(class_='resultsimilarityinfo').text
		similarity = float(similarity_str[:-1])
		thumbnail = result.find(class_='resultimage').find('img').get('src')
		title = result.find(class_='resulttitle').text
		print(thumbnail)
		content = result.find(class_='resultcontentcolumn')
		site = 'None'
		if content.find('strong'):
			site = content.find('strong').text
		TV = 'None'
		ID = 'None'
		ID_link = 'None'
		author = 'None'
		author_link = 'None'
		if site=='None':
			pass
		elif site=='Title: ':
			site = 'None'
			TV = content.text
		else:
			site=site[:-5]
			info = content.find_all(class_='linkify')
			if len(info)!=0:
				try:
					ID = info[0].text
					ID_link = info[0].get('href')
				except:
					ID = ''
					ID_link = ''
				try:
					author = info[1].text
					author_link = info[1].get('href')
				except:
					author = ''
					author_link = ''
		pack={'similarity_str':similarity_str, 'similarity':similarity, 'thumbnail':thumbnail, 'title':title, 'site':site,'TV':TV, 'ID':ID, 'author':author, 'ID_link':ID_link, 'author_link':author_link}
		package.append(pack)
	return package

def search(url):
	global login_res
	if not login_res:
		login_res = login()
	res = s.get(f"{search_url}{url}", headers=header)
	if res.status_code != 200:
		login()
		res = s.get(search_url+url, headers=header)
	page = bs(res.text,'lxml')
	results = page.find_all(class_ = 'resulttable')
	if len(results)==0:
		return -1
	package = []
	for result in results:
		similarity_str = result.find(class_='resultsimilarityinfo').text
		similarity = float(similarity_str[:-1])
		thumbnail = result.find(class_='resultimage').find('img').get('src')
		if thumbnail == 'images/static/blocked.gif':
			thumbnail = result.find(class_='resultimage').find('img').get('data-src')
		title = result.find(class_='resulttitle').text
		print(thumbnail)
		rcc = result.find_all(class_='resultcontentcolumn')
		pack = {
			'similarity_str': similarity_str,
			'similarity': similarity,
			'title': title,
			'thumbnail': thumbnail,
			'info': []
		}
		tmp = ['',[]]
		for col in rcc:
			for c in col.children:
				if c.name == 'strong':
					if tmp[0]:
						pack['info'].append(tmp)
					tmp = [c.text,[]]
				else:
					if not c.text:
						pass
					elif c.name == 'a' and 'href' in c.attrs:
						tmp[1].append(f'{c.text} {c.attrs["href"]}')
					else:
						tmp[1].append(f'{c.text}')
		if tmp[0]:
			pack['info'].append(tmp)
		package.append(pack)
	return package


import subprocess
import json
import sys
import simplim
import re
import os

module_path = os.path.join(os.path.dirname(__file__), '')

temp_path = os.path.join(module_path, 'temp')

class App():
	def __init__(self, appn, hous, mins, secs, pcnt):
		self.appn = appn
		self.hous = hous
		self.mins = mins
		self.secs = secs
		self.pcnt = pcnt
		self.allT = hous * 3600 + mins * 60 + secs


def call(cmd):
	apps = []
	subp = subprocess.Popen(cmd, shell = True, stdout = subprocess.PIPE)
	time = subp.stdout.readline().decode()
	line = subp.stdout.readline().decode()
	line = subp.stdout.readline().decode()
	while line:
		print(line)
		# cuts = re.search('    ', line[10:])
		# appn = line[re.search('\S',line[:10 + cuts.start()]).start():10 + cuts.start()]
		nsta = re.search('\S',line).start()
		nend = nsta + re.search('\ ?\ ?\ ?\ ?\ ?\ ?\ ?\ ?\ ?\d?\d?h? ?\d?\d?m? ?\d\ds', line[nsta:]).start()
		
		appn = line[nsta:nend]
		info = line[nend:]

		secs = int(re.search('\d\ds', info)[0][:-1])
		mins = re.search('\d\dm', info)
		if mins:
			mins = int(mins[0][:-1])
		else:
			mins = 0
		hous = re.search('\d?\dh', info)
		if hous:
			hous = int(hous[0][:-1])
		else:
			hous = 0
		pcnt = re.search('(\d)(\d)?\.\d%', info)[0]

		apps.append(App(appn, hous, mins, secs, pcnt))
		line = subp.stdout.readline().decode()
	return apps

def out(cmd, clean=1):
	temp = os.path.join(temp_path, simplim.gettemp() + '.json')
	os.system(cmd + ' -O ' + temp)
	list_ = json.load(open(temp))
	list_out = []
	for dict_ in list_:
		list_out.append(simplim.easyclassify(dict_))
	if clean:
		cleantemp()
	return list_out

def cleantemp():
	os.system('sudo rm -r ' + temp_path)
	os.system('mkdir ' + temp_path)




r"""@2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool using api <https://v1.hitokoto.cn> to get sentence from hitokoto
api doc: <https://developer.hitokoto.cn/sentence/>
"""

import requests as rq

host = 'https://international.v1.hitokoto.cn'
cn_host = 'https://v1.hitokoto.cn'
c_ = {
	'a': '动画',
	'b': '漫画',
	'c': '游戏',
	'd': '文学',
	'e': '原创',
	'f': '来自网络',
	'g': '其他',
	'h': '影视',
	'i': '诗词',
	'j': '网易云',
	'k': '哲学',
	'l': '抖机灵'
}

def cn_call(c = '', encode = 'json', charset = '', callback = '', select = ''):
	params = {
		'c': c,
		'encode': encode,
		'charset': charset,
		'callback': callback,
		'select': select
	}
	response = rq.get(url = cn_host, params = params).json()
	text = response['hitokoto']
	type_ = c_[response['type']]
	from_ = response['from']
	return text, type_, from_

def call(c = '', encode = 'json', charset = '', callback = '', select = ''):
	params = {
		'c': c,
		'encode': encode,
		'charset': charset,
		'callback': callback,
		'select': select
	}
	response = rq.get(url = host, params = params).json()
	text = response['hitokoto']
	type_ = c_[response['type']]
	from_ = response['from']
	return text, type_, from_


from matplotlib import pyplot as plt
from io import BytesIO
from PIL import Image, ImageDraw, ImageFont
import numpy as np 
import base64
import cv2
import os

module_path = os.path.join(os.path.dirname(__file__), '')

def raw2arr(raw, rgb=0):
	arr = cv2.imdecode(np.frombuffer(raw, np.uint8), cv2.IMREAD_ANYCOLOR)
	if rgb:
		b,g,r = cv2.split(arr)  
		arr = cv2.merge([r,g,b])
	return arr

def arr2raw(arr, rgb=0):
	if rgb:
		r,g,b,a = cv2.split(arr)  
		arr = cv2.merge([b,g,r])
	return cv2.imencode('.jpg', arr)[1].tobytes()


def raw2base64(raw):

	return base64.b64encode(raw).decode()

def any2raw(img, type_):
	if type_ == 'base64':
		return base64.b64decode(img)
	elif type_ == 'arr':
		return cv2.imencode('.jpg', img)[1].tobytes()
	elif type_ == 'plt':
		fig = plt.figure()
		cvs = fig.canvas
		cvs.print_png(img)

def gray4raw(raw):
	img_cv = raw2arr(raw)
	img_cv_gray = cv2.cvtColor(img_cv, cv2.COLOR_BGR2GRAY)
	return arr2raw(img_cv_gray)

def resize(arr, size):
	pass

def avglight(array, amount):
	for row in array:
		for pixel in row:
			rate = np.max(pixel[:3]) / amount
			if rate:
				pixel[0]/=rate
				pixel[1]/=rate
				pixel[2]/=rate
			else:
				pixel[0]+=amount
				pixel[1]+=amount
				pixel[2]+=amount
	return array

def draw(path, text, pos=(0, 0), font = os.path.join("Fonts", "STXIHEI.ttf"), size = 18, color='#000000'):
	img = Image.open(path)
	draw = ImageDraw.Draw(img)
	font = ImageFont.truetype(font, size)
	draw.text(pos, text, font = font, fill = color)
	return img







r"""©2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool simplified many useful functions
"""

from io import BytesIO
import requests as requests
import threading
import random
import shutil
import time
import sys
import os
import re

module_path = os.path.join(os.path.dirname(__file__), '')

class sim():
	def __init__(self):
		super(sim, self).__init__()
		self._running = os.path.dirname(sys.argv[0])

class DownloadBar():
	def __init__(self):
		self.total = 0
		self.left = 0
		self.percent = 0
		self.speed_Mb = 0
		self.speed_b = 0

class MultiThreadDownloader:
	"""docstring for MultiThreadDownloader"""
	def __init__(self, url, headers, segments, path):
		self.url = url
		self.headers = headers
		self.segments = segments
		if not os.path.exists(path):
			self.fp = open(path,'wb')
			self.fp.close()
		else:
			ans = input('>>> File exists, continue?[y/*]')
			if ans == 'y':
				self.fp = open(path,'wb')
				self.fp.close()
			else:
				return
		self.fp = open(path,'rb+')
		self.size = requests.head(url, headers = headers).headers['content-length']
		self.size = int(self.size)
		print(self.size)
	def setPos(self):
		spos = []
		fpos = []
		persize = int(self.size / self.segments)
		intsize = persize * self.segments  # 整除部分

		for i in range(0, intsize, persize):
			spos.append(i)
			fpos.append(i + persize - 1)
		if intsize < self.size:	 # size 不一定 n 等分，将不能等分余下的部分添加到最后一个 sub 里
			fpos[self.segments - 1] = self.size
		
		self.spos = spos
		self.fpos = fpos
		print(spos, '\n',fpos)
	def downloadFile(self, spos, fpos):
		print('down')
		try:
			headers = {}
			headers["Range"] = "bytes=%d-%d" % (spos, fpos)
			for key in self.headers:
				headers[key] = self.headers[key]
			res = requests.get(self.url, headers = headers)
			self.fp.seek(spos)
			self.fp.write(res.content)
		except Exception as e:
			print ("downloadFile error: ", e)
	def run(self):
		self.setPos()
		tmp = []

		start_time = time.time()	# 计时起始点，为了计算平均速度

		for i in range(0, self.segments):
			t = threading.Thread(target = self.downloadFile, args=(self.spos[i], self.fpos[i]))
			t.setDaemon(True)	   # 主进程结束时，线程也随之结束
			t.start()
			tmp.append(t)
		sleep(1)
		for i in tmp:
			i.join()				# 等待线程结束
		self.fp.close()

		finish_time = time.time()   # 计时终点，为了计算平均速度
		delta_time = finish_time-start_time
		speed = float(self.size)/( 1000.0*(finish_time-start_time))
		print ( "spend time: %0.2f s" % float(finish_time-start_time) )
		print ( "finished... average speed: %0.2f KB/s" % speed )
		return self.size, delta_time, speed

def multiThreadDownload(url, headers, path, chunk_size=32, verify=True):
	headers['keep-alive']='close'
	res = requests.head(url, headers = headers, verify=verify)
	trying_times=0
	while 1:
		try:
			trying_times+=1
			print(trying_times)
			if trying_times==5:
				return False
			size = int(res.headers["Content-Length"])
			break
		except:
			res = requests.get(url, headers = headers, stream=True, verify = verify)
			continue
	res = requests.head(url, headers = headers, verify=verify)
	print ( "total size: %d" % size )
	n=int(size/1024/chunk_size)
	if not n:
		n = 1
	spos = []
	fpos = []
	persize = int(size/n)
	intsize = persize * n  # 整除部分

	for i in range(0, intsize, persize):
		spos.append(i)
		fpos.append(i+persize-1)
	if intsize < size:	 # size 不一定 n 等分，将不能等分余下的部分添加到最后一个 sub 里
		fpos[n-1] = size
	print ( spos )
	print ( fpos )
	global cnt
	cnt = 0
	def downloadFile(url, spos, fpos, fp):
		global cnt
		while 1:
			try:
				header = {}
				header["Range"] = "bytes=%d-%d" % (spos, fpos)
				for key in headers:
					header[key] = headers[key]
				res = requests.get(url, headers = header, timeout=10,verify=verify)
				fp.seek(spos)
				fp.write(res.content)
				print(fpos)
				cnt+=1
				print(cnt,'/',n)
				return
			except Exception as e:
				# print ("downloadFile error: ", e)
				print(fpos,'retry')
				continue

	fp = open(path, "wb")
	fp.close()
	fp = open(path, "rb+")
	tmp = []

	start_time = time.time()	# 计时起始点，为了计算平均速度

	for i in range(0, n):
		t = threading.Thread(target = downloadFile, args=(url,  spos[i], fpos[i], fp))
		t.setDaemon(True)	   # 主进程结束时，线程也随之结束
		t.start()
		tmp.append(t)
	for i in tmp:
		i.join()
						# 等待线程结束
	fp.close()

	finish_time = time.time()   # 计时终点，为了计算平均速度

	speed = float(size)/( 1000.0*(finish_time-start_time))
	print ( "spend time: %0.2f s" % float(finish_time-start_time) )
	print ( "finished... average speed: %0.2f KB/s" % speed )
	return True


def logo():
	logo = '╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM\n┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸\n╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸'
	print(logo)

def LOGO():
	logo = '██╗  ██╗ █████╗ ███████╗██████╗ ███████╗███████╗██╗	 ██╗███████╗███████╗\n██║  ██║██╔══██╗██╔══██║██╔══██╗██╔══██║██╔════╝██║	 ██║██╔════╝██╔════╝\n███████║███████║███████║██║  ██║███████║███████╗██║	 ██║██║	 ███████╗\n██╔══██║██╔══██║██╔═██╔╝██║  ██║██╔═██╔╝██╔════╝██║	 ██║██║	 ██╔════╝\n██║  ██║██║  ██║██║  ██╗██████╔╝██║  ██╗███████╗███████╗██║███████╗███████╗\n╚═╝  ╚═╝╚═╝  ╚═╝╚═╝  ╚═╝╚═════╝ ╚═╝  ╚═╝╚══════╝╚══════╝╚═╝╚══════╝╚══════╝'
	print(logo)

def say(x,*arg):
	if len(arg)==0:
		print('>>>', x)
	else:
		string = ''
		for y in arg:
			string += str(y)+' '
		print('>>>', x, string)

def hear(x):

	return input('>>> '+x)

def clear():

	os.system('clear')

def curl(path, name ,url):
	os.system('curl -o '+os.path.join(path, name)+' '+url)

def download(url, headers = {}, delta_time = 1, chunk_size = 8, ratio = 1000, object_ = None, need_content_length = True):
	down = requests.get(url, stream = True, headers = headers)
	if not need_content_length:
		f=BytesIO()
		f.write(down.content)
		return f
	trying_times = 0
	while 1:
		print(trying_times)
		if trying_times > 10:
			return 0
		try:
			length = down.headers['content-length']
			print(trying_times)
			break
		except:
			trying_times += 1
			down = requests.get(url, stream = True, headers = headers)
	print(down.headers)
	length = float(length)
	Kb = length / ratio
	Mb = Kb / ratio
	Mb_str = '{:.2f}'.format(Mb)
	done = 0
	done_pre = 0
	time_pre = time.time()
	f = BytesIO()
	for chunk in down.iter_content(chunk_size = chunk_size):
		if chunk:
			f.write(chunk)
			f.flush()
			done += len(chunk)
			if time.time() - time_pre > delta_time:
				percent = done / length * 100
				num = int(percent / 2)
				speed_b = (done - done_pre)/delta_time
				speed_Mb = (done - done_pre) / ratio / ratio / delta_time
				done_pre = done
				left = str(int((length - done)/speed_b))
				if object_:
					object_.percent = percent
					object_.total = Mb
					object_.speed_Mb = speed_Mb
					object_.speed_b = speed_b
					object_.left = left
				print('\r' + num*'█' + (50-num)*' ' + '{:.2f}'.format(percent) + '% ' + str(speed_Mb)[:4] + 'M/s ' + Mb_str +'M ' + left + 's   ',end='',flush=True)
				time_pre = time.time()
	# print('\n[N]', 'successfully downloaded')
	return f

def match(arg, arr):
	'''
	match(arg, arr) -> bool
	'''
	for opt in arr:
		if arg == opt[0]:
			return True
	return False

def findict_0(dict_, arr):
	'''
	dict={'1':{'2':{'3':'got it!'}}}
	dict=findict(dict,['1','2','3']) -> dict: 'got it!'
	'''
	if len(arr) != 0:
		dict_ = dict_[arr[0]]
		arr = arr[1:]
		dict_ =  findict(dict_,arr)
		return dict_
	else:
		return dict_

def findict(dict_, arr):
	if len(arr) != 0:
		if type(dict_) == type({'1':'1'}) :
			for key in dict_:
				if key == arr[0]:
					dict_ = dict_[arr[0]]
					arr  = arr[1:]
					dict_ = findict(dict_,arr)
					return dict_
			return 'Keyword not found!'
		else:
			return 'Keyword not found!'
	else:
		return dict_

def filtemoji(desstr, restr=''):
	try:
		co = re.compile(u'[\U00010000-\U0010ffff]')
	except re.error:
		co = re.compile(u'[\uD800-\uDBFF][\uDC00-\uDFFF]')
	return co.sub(restr, desstr)


def getcmd(cmd):
	which = -1
	cmdarr = []
	for char in range(len(cmd)):
		if cmd[char] != ' ' and char == 0:
			which+=1
			cmdarr.append('')
		if cmd[char] != ' ':
			cmdarr[which]+=cmd[char]
		if char == len(cmd)-1:
			break
		if cmd[char] == ' ' and cmd[char+1] != ' ':
			which+=1
			cmdarr.append('')
	if which == -1:
		return []
	return cmdarr

def parsedict(dict_):
	keys = []
	values = []
	for key in dict_:
		if type(dict_[key]) == type(dict_):
			keys.append(key)
		else:
			values.append(key)
	return keys, values

def checkdoc(list_, doc):
	for dir_ in list_:
		if dir_==doc:
			return 1
	return 0

def localtime(setting = 0):
	lt = time.localtime()
	if not setting:
		return lt
	else:
		return lt[:setting]

def tuple2str(tups, isPure = 1):
	string=''
	if isPure:
		for tup in tups:
			string+=str(tup)
		return string
	else:
		pre = str(tups)
		for p in pre:
			if p != '\'':
				string += p
		return string

def list2str(list_, spliter = ''):
	string = ''
	for item in list_:
		string += str(item) + spliter
	if spliter:
		return string[:-len(spliter)]
	return string

def listsum(*list_):
	List = []
	length = 0
	for sublist in list_:
		if len(sublist) > length:
			length = len(sublist)
	for ele in range(length):
		List.append(0)
	for x in range(length):
		for sublist in list_:
			if len(sublist) > x:
				List[x] += sublist[x]
	return List

def standardfile(file):
	sfile = ''
	for f in file:
		if f == '/':
			sfile += ':'
		elif f == ':':
			sfile += '：'
		else:
			sfile += f
	return sfile

def standardnum(num, length=2):
	l = len(str(num)) 
	if l < length:
		return '0' * (length - l) + str(num)
	else:
		return str(num)

def gettemp(setting=6):
	lt = localtime(setting)
	# print(lt)
	new_lt = []
	for t in lt:
		new_lt.append(standardnum(t,2))
	# print(new_lt)
	return list2str(new_lt)

def getargs(function):

	return function.__code__.co_varnames[:function.__code__.co_argcount]

def getargsnum(function):

	return function.__code__.co_argcount

def getargsdefault(function):
	
	return function.__defaults__

def classify(dict_, depth = 0, arr = [], key = 0):
	keys, values = parsedict(dict_)
	arr.append((depth, keys, values))
	if keys:
		depth += 1
		for key in keys:
			newdict_ = dict_[key]
			classify(newdict_, depth, arr, key)
	return arr

def classify2(dict_, depth = 0, key = 'main', type_ = 1, path = [], note = []):
	keys, values = parsedict(dict_)
	#path.append((depth, keys, values))
	note.append((depth, key, type_))
	if keys:
		depth += 1
		for key in keys:
			newdict_ = dict_[key]
			classify2(newdict_, depth, key, 1, path)
	if values:
		if not keys:
			depth += 1
		for value in values:
			note.append((depth, value, 0))
	return note

def addself(tab, key, value):
	if type(value) == type(''):
		return tab*'\t' + 'self.' + key + ' = \'\'\'' + value + '\'\'\'\n'
	else:
		return tab*'\t' + 'self.' + key + ' = ' + str(value) + '\n'

def deffunc(tab, name, args, funcarr):
	string = tab * '\t' + 'def ' + name + tuple2str(args, isPure = 0) + ':\n'
	for func in funcarr:
		try:
			intab = func[1]
		except:
			intab = 0
		try:
			cmd = func[0]
		except:
			cmd = func
		string += (tab + 1 + intab) * '\t' + cmd + '\n'
	return string

def easyclassify_arr(dict_, arr = []):
	keys, values = parsedict(dict_)
	if keys:
		for key in keys:
			newdict_ = dict_[key]
			easyclassify_arr(newdict_)
	if values:
		for value in values:
			arr.append((value,dict_[value]))
	return arr

def easyclassify(dict_):
	string = 'class Classify:\n\tdef __init__(self):\n'
	arr = easyclassify_arr(dict_)
	for a in arr:
		string += addself(2, a[0], a[1])
	try:
		os.mkdir('classify_temp')
		fp = open('classify_temp/__init__.py','w')
		fp.write(string)
		fp.close()
		import classify_temp
		shutil.rmtree('classify_temp')
		return classify_temp.Classify()
	except Exception as e:
		print('[E]','Failed to classify')
		raise e
#depth, key, type_ 0:value, 1:dict_

def rhash(value):
	if type(value) == type(''):
		length = len(value)
		tot = 0
		HashMap = ''
		Hash = ''
		pos = 0
		for var in value:
			h = hex(int(oct(int(bin((ord(var) + length)**3)[2:]))[2:]))[2:]
			tot += ord(var)
			HashMap += h
		avg = int(tot/length)
		size = len(HashMap)
		if size < 16:
			HashMap += (16-size) * 'k'
			size = 16
		# print('Map',HashMap)
		newHash = ''
		for i in range(0,16):
			num = 0
			for j in range(0,int(size/16)):
				num += ord(HashMap[j*16+i])
			newHash += chr(int(num*16/size))
		bias = tot % 27 + avg % 11
		for H in newHash:
			if pos%2 == 0:
				char = ord(H)+bias
				if char>=127 and char<=160:
					char += 34
				Hash += chr(char)
			pos += 1
		# print('Hash',Hash)
		return Hash
	if type(value) == type(0) or type(0.1):
		return value















































def MakeTag(tag, value, style = ''):
	return '<' + tag + ' style=\" ' + style + '\">' + value + '</'+tag+'>' 

def MakeHtml(head, body):
	return '<!DOCTYPE html><html>' + head + body + '</html>'

def MakeHead(dict_ = {}, title = ''):
	head = MakeTag('title', title)
	if dict_:
		for key in dict_:
			head += MakeTag(key, dict_[key])
	return MakeTag('head', head)

def AddUp(dict_, br = ''):
	result = ''
	if br:
		br = '<br>'
	for tag in dict_:
		result += tag + br
	return result

def MakeBody(dict_ = {}):
	body = ''
	if dict_:
		for key in dict_:
			body += MakeTag(key, dict_[key][0], dict_[key][1]) + '<br>'
	return MakeTag('body', body)
	
def Html(title = '', head_dict = {}, body_dict = {}, path = None):
	html = MakeHtml(MakeHead(head_dict, title), MakeBody(body_dict))
	if path:
		open(path,'w').write(html)
	else:
		return html

def SimpleHtml(head_dict = {}, body_dict = {}, path = None):
	html = MakeHtml(AddUp(head_dict), AddUp(body_dict, 1))
	if path:
		open(path,'w').write(html)
	else:
		return html


	


r"""@2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool made for downloading comics from dmzj.com
"""

from bs4 import BeautifulSoup as bs
import simplim.api.translate.youdao as yd
from selenium import webdriver
from pathlib import Path
import requests as rq
import pypinyin
import random
import json
import time
import simplim
import os

module_path = __file__[:-11]

testurl = 'https://images.dmzj.com/b/%E5%88%AB%E5%BD%93%E6%AC%A7%E5%B0%BC%E9%85%B1%E4%BA%86/%E7%AC%AC1%E8%AF%9D_1491547130/01.jpg'

host	= 'https://manhua.dmzj.com/'

referer = 'https://manhua.dmzj.com/'

headers = {
	'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36',
	'referer': referer
}

class DMZJ:
	def __init__(self):		
		self.option = webdriver.ChromeOptions()
		self.option.add_argument("headless")
		self.driver = None
		self.quited = 1
	def load(self):
		if self.quited:
			self.driver = webdriver.Chrome(options = self.option)
			self.quited = 0
	def quit(self):
		if not self.quited:
			self.driver.quit()
			self.quited = 1

Dmzj = DMZJ()

def getIdByName(name, full = 0):
	id_ = ''
	for i in pypinyin.pinyin(name, style=pypinyin.NORMAL):
		if full:
			id_ += ''.join(i)[0]
		else:
			id_ += ''.join(i)
	return id_

def getImg(url, name = 0, path = './', type_ = 'jpg', useCurl = 0 ,ui = 0):
	url = 'https:'+url
	if not name:
		name_ = str(simplim.tuple2str(simplim.localtime(6)))
	else:
		name_ = str(name)
	path_ = os.path.join(path, simplim.standardfile(name_) + '.' + type_)
	if not Path(path).exists():
		os.mkdir(path)
		print('[N] mkdir successed')
	if useCurl:
		os.system('curl --header "referer: '+ referer +'" -o '+ path_ + ' ' + url)
	else:
		if ui:
			f = simplim.download(url, headers, ui)
		else:
			f = rq.get(url, headers=headers).content
		print('\n','[N]',str(name_) + '.' + type_, 'successfully downloaded')
		open(path_, 'ab').write(f)

def getInfo(url):
	# try:
	Dmzj.driver.get(url)
	page = bs(Dmzj.driver.page_source, 'lxml')
	div = page.find('div', class_='cartoon_online_border')
	ul = div.find_all('li')
	info = []
	for li in ul:
		dict_ = dict(name = li.text, url = li.find('a').get('href'))
		info.append(dict_)
	return info
	# except:
		# print('[E]','Failed to getInfo')

def getPicUrl(url):
	url = host + url + '#@page=1'
	try:
		Dmzj.driver.get(url)
		options = bs(Dmzj.driver.page_source, 'lxml').find_all('option')
		urls = []
		for option in options:
			urls.append(option.get('value'))
		return urls
	except:
		print('[E]','Failed to get urls')

def getComic(name, id_ = None, path = './', type_ = 'jpg', useCurl = 0 ,ui = 0, chapter = 0):
	Dmzj.load()
	path_ = os.path.join(path, name)
	if id_:
		url = host + id_
		if rq.get(url).status_code != 200:
			print('[N]','Comic Not Found')
			Dmzj.quit()
			return
	else:
		if rq.get(host + getIdByName(name, 0)).status_code == 200:
			url = host + getIdByName(name, 0)
		elif rq.get(host + getIdByName(name, 1)).status_code == 200:
			url = host + getIdByName(name, 1)
		else:
			print('[N]','Comic Not Found')
			Dmzj.quit()
			return
	if not os.path.exists(path_):
		os.mkdir(path_)
	info = getInfo(url)
	if not chapter:
		for inf in info:
			urls = getPicUrl(inf['url'])
			path__ = os.path.join(path_, inf['name'])
			page = 1
			for url in urls:
				getImg(url, name = page, path = path__, type_ = type_, useCurl = useCurl ,ui = ui)
				page += 1
	else:
		try:
			print('try')
			ans = input('>>>',info[int(chapter)]['name'],'y/n?')
			if ans == 'y':
				path__ = os.path.join(path_, inf['name'])
				page = 1
				for url in urls:
					getImg(url, name = page, path = path__, type_ = type_, useCurl = useCurl ,ui = ui)
			else:
				return
		except:
			print('except')
			try:
				for inf in info:
					if re.search(chapter, inf['name']):
						ans = input('>>>',inf['name'],'y/n?')
						if ans == 'y':
							path__ = os.path.join(path_, inf['name'])
							page = 1
							for url in urls:
								getImg(url, name = page, path = path__, type_ = type_, useCurl = useCurl ,ui = ui)
							break
						else:
							return
			except:
				print('[E]','chapter error')
	# finally:
	Dmzj.quit()

def quit():
	Dmzj.quit()

def load():
	Dmzj.load()










r"""@2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

headers for many website
"""

module_path = __file__[:-11]

agent = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36'

referers = {
	'None': '',
	'pixiv': 'https://www.pixiv.net/',
	'dmzj': 'https://manhua.dmzj.com/',
	'baidu': 'https://www.baidu.com/',
	'bili': 'https://www.bilibili.com'
}


def get(site ,agent = agent):
	headers = {
		'user-agent': agent,
		'referer':  referers[site]
	}
	return headers

# header = {
#     "User-Agent": 'Mozilla/5.0 (compatible; Baiduspider/2.0; +http://www.baidu.com/search/spider.html)',
# }
# 
# Baidu_grab = requests.get(url, headers=header)


from bs4 import BeautifulSoup as bs
import requests as rq
import re, json, simplim, os
readOnline = 'https://ahentai.top/index.php?route=comic/readOnline&comic_id='
galleryPage = 'https://ahentai.top/index.php?route=comic/article&comic_id='
import urllib3
urllib3.disable_warnings()
def Download(id_, path=''):
	id_ = str(id_)
	text = rq.get(readOnline+id_).text
	lxml = bs(text, 'lxml')
	name=lxml.find(id="content").find(class_='d').text
	if not path:
		path = name
	if not os.path.exists(path):
		try:
			os.mkdir(path)
		except:
			print('Invalid path!')
			return
	open(os.path.join(path,'id'),'w').write(id_)
	scripts = lxml.find_all('script')
	Image_List = ''
	for script in scripts:
		if re.search('Image_List', script.text):
			Image_List = script.text
	print(Image_List)
	host = 'https:' + re.search('\"(.*?)\"', Image_List).group(0)
	pattern = re.compile(r'Image_List = \[(.*?)\]')
	Image_List = json.loads(pattern.search(Image_List).group(0)[13:])
	pics = len(Image_List)
	try:
		for Image in Image_List:
			url = host+Image['sort']+'.jpg'#Image['extension']
			if not os.path.exists(os.path.join(path,Image['sort']+'.'+'jpg')):
				simplim.curl(path, Image['sort']+'.'+'jpg' , url)
	except:
		return

def multiDownload(id_, path=''):
	id_ = str(id_)
	text = rq.get(readOnline+id_).text
	lxml = bs(text, 'lxml')
	name=lxml.find(id="content").find(class_='d').text
	if not path:
		path = name.replace('/',':')
	if not os.path.exists(path):
		try:
			os.mkdir(path)
		except:
			print('Invalid path!')
			return
	open(os.path.join(path,'id'),'w').write(id_)
	scripts = lxml.find_all('script')
	Image_List = ''
	for script in scripts:
		if re.search('Image_List', script.text):
			Image_List = script.text
	print(Image_List)
	host = 'https:' + re.search('\"(.*?)\"', Image_List).group(0)[1:-1]
	pattern = re.compile(r'Image_List = \[(.*?)\]')
	Image_List = json.loads(pattern.search(Image_List).group(0)[13:])
	pics = len(Image_List)
	try:
		for Image in Image_List:
			print(Image['sort'], '/', pics)
			url = host+Image['sort']+'.jpg'#Image['extension']
			if not os.path.exists(os.path.join(path,Image['sort']+'.'+'jpg')):
				simplim.multiThreadDownload(url = url, headers = {}, path = os.path.join(path, Image['sort']+'.'+'jpg'),verify=False,chunk_size=32)
	except Exception as e:
		raise e
		return




r"""@2022 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

 simple wrap for setu api <https://api.lolicon.app/setu/v2>
"""

'''
r18	int	0	0为非 R18，1为 R18，2为混合（在库中的分类，不等同于作品本身的 R18 标识）
num	int	1	一次返回的结果数量，范围为1到100；在指定关键字或标签的情况下，结果数量可能会不足指定的数量
uid	int[]		返回指定uid作者的作品，最多20个
keyword	string		返回从标题、作者、标签中按指定关键字模糊匹配的结果，大小写不敏感，性能和准度较差且功能单一，建议使用tag代替
tag	string[]		返回匹配指定标签的作品，详见下文
size	string[]	["original"]	返回指定图片规格的地址，详见下文
proxy	string	i.pixiv.cat	设置图片地址所使用的在线反代服务，详见下文
dateAfter	int		返回在这个时间及以后上传的作品；时间戳，单位为毫秒
dateBefore	int		返回在这个时间及以前上传的作品；时间戳，单位为毫秒
dsc	boolean	false	设置为任意真值以禁用对某些缩写keyword和tag的自动转换，详见下文

dsc:
vtb => 虚拟YouTuber|VTuber
fgo => Fate/GrandOrder|Fate/Grand Order|FateGrandOrder
pcr => 公主连结|公主连结Re:Dive|プリンセスコネクト
gbf => 碧蓝幻想
舰b => 碧蓝航线|AzurLane
舰c => 舰队collection
少前 => 少女前线|girlsfrontline

respond
pid	int	作品 pid
p	int	作品所在页
uid	int	作者 uid
title	string	作品标题
author	string	作者名（入库时，并过滤掉 @ 及其后内容）
r18	boolean	是否 R18（在库中的分类，不等同于作品本身的 R18 标识）
width	int	原图宽度 px
height	int	原图高度 px
tags	string[]	作品标签，包含标签的中文翻译（有的话）
ext	string	图片扩展名
uploadDate	int	作品上传日期；时间戳，单位为毫秒
urls	object	包含了所有指定size的图片地址
'''

import requests as rq

host = "https://api.lolicon.app/setu/v2"

params = {
	"r18": 0,
	"num": 1,
	"uid": "",
	"keyword": "",
	"tag": "",
	# "size": ["original","regular","small","thumb","mini"],
	"size": "original",
	"proxy": "i.pximg.net",
	"dateAfter": "",
	"dateBefore": "",
	# "dsc": "vtb|fgo|pcr|gbf|舰b|舰c|少前",
	"dsc": "",
}

def call(params):
	return rq.get(host,params=params)



import threading
import requests
import time
import sys
import os

class MulThreadDownload(threading.Thread):
	def __init__(self,url,startpos,endpos,f):
		super(MulThreadDownload,self).__init__()
		self.url = url
		self.startpos = startpos
		self.endpos = endpos
		self.fd = f

	def download(self):
		print("start thread:%s at %s" % (self.getName(), time.time()))
		headers = {"Range":"bytes=%s-%s"%(self.startpos, self.endpos)}
		res = requests.get(self.url, headers = headers)
		# res.text 是将get获取的byte类型数据自动编码，是str类型， res.content是原始的byte类型数据
		# 所以下面是直接write(res.content)
		self.fd.seek(self.startpos)
		self.fd.write(res.content)
		print("stop thread:%s at %s" % (self.getName(), time.time()))
		# f.close()

	def run(self):
		self.download()

def download(name, type_, threadnum = 3):
	name = name + '.' + type_
	filesize = int(requests.head(url).headers['Content-Length'])
	threading.BoundedSemaphore(threadnum)
	step = filesize // threadnum
	mtd_list = []
	start = 0
	end = -1
	tempf = open(name,'w')
	tempf.close()
	# rb+ ，二进制打开，可任意位置读写
	with open(name,'rb+') as  f:
		fileno = f.fileno()
		# 如果文件大小为11字节，那就是获取文件0-10的位置的数据。如果end = 10，说明数据已经获取完了。
		while end < filesize -1:
			start = end +1
			end = start + step -1
			if end > filesize:
				end = filesize
			# print("start:%s, end:%s"%(start,end))
			# 复制文件句柄
			dup = os.dup(fileno)
			# print(dup)
			# 打开文件
			fd = os.fdopen(dup,'rb+',-1)
			# print(fd)
			t = MulThreadDownload(url,start,end,fd)
			t.start()
			mtd_list.append(t)
		for i in  mtd_list:
			i.join()


import numpy as np  
from time import time

def boxmuller():
	sigma = 1
	size = 1
	u = np.random.uniform(size=size)  
	v = np.random.uniform(size=size)  
	z = np.sqrt(-2 * np.log(u)) * np.cos(2 * np.pi * v)  
	return (z * sigma)[0]

class Mersenne:
	def __init__(self):
		self.index = 624
		self.MT = [0]*self.index
	def inter(self, t):
		return(0xFFFFFFFF & t) #取最后32位->t

	def twister(self):
		for i in range(624):
			y = self.inter((self.MT[i] & 0x80000000) +(self.MT[(i + 1) % 624] & 0x7fffffff))
			self.MT[i] = self.MT[(i + 397) % 624] ^ y >> 1
			if y % 2 != 0:
				self.MT[i] = self.MT[i] ^ 0x9908b0df
		self.index = 0

	def exnum(self):
		if self.index >= 624:
			self.twister()
		y = self.MT[self.index]
		y = y ^ y >> 11
		y = y ^ y << 7 & 2636928640
		y = y ^ y << 15 & 4022730752
		y = y ^ y >> 18
		self.index = self.index + 1
		return self.inter(y)

	def mainset(self,seed):
		self.MT[0] = seed	#seed
		for i in range(1,624):
			self.MT[i] = self.inter(1812433253 * (self.MT[i - 1] ^ self.MT[i - 1] >> 30) + i)
		return self.exnum()

	def twist(self,mi,ma):
		so = self.mainset(int(time())) / (2**32-1)
		rd = mi + int((ma-mi)*so)
		return rd



import random
import string
import time

from httpx import AsyncClient

from get_data import old_version_get_ds_token, random_hex

# 米游社的API列表
bbs_Cookieurl = 'https://webapi.account.mihoyo.com/Api/cookie_accountinfo_by_loginticket?login_ticket={}'
bbs_Cookieurl2 = 'https://api-takumi.mihoyo.com/auth/api/getMultiTokenByLoginTicket' \
                 '?login_ticket={}&token_types=3&uid={}'
bbs_Taskslist = 'https://bbs-api.mihoyo.com/apihub/sapi/getUserMissionsState'  # 获取任务列表
bbs_Signurl = 'https://bbs-api.mihoyo.com/apihub/sapi/signIn?gids={}'  # post
bbs_Listurl = 'https://bbs-api.mihoyo.com/post/api/getForumPostList?' \
              'forum_id={}&is_good=false&is_hot=false&page_size=20&sort_type=1'
bbs_Detailurl = 'https://bbs-api.mihoyo.com/post/api/getPostFull?post_id={}'
bbs_Shareurl = 'https://bbs-api.mihoyo.com/apihub/api/getShareConf?entity_id={}&entity_type=1'
bbs_Likeurl = 'https://bbs-api.mihoyo.com/apihub/sapi/upvotePost'  # post json 

mihoyobbs_List = [{
    'id'     : '1',
    'forumId': '1',
    'name'   : '崩坏3',
    'url'    : 'https://bbs.mihoyo.com/bh3/'
}, {
    'id'     : '2',
    'forumId': '26',
    'name'   : '原神',
    'url'    : 'https://bbs.mihoyo.com/ys/'
}, {
    'id'     : '3',
    'forumId': '30',
    'name'   : '崩坏2',
    'url'    : 'https://bbs.mihoyo.com/bh2/'
}, {
    'id'     : '4',
    'forumId': '37',
    'name'   : '未定事件簿',
    'url'    : 'https://bbs.mihoyo.com/wd/'
}, {
    'id'     : '5',
    'forumId': '34',
    'name'   : '大别野',
    'url'    : 'https://bbs.mihoyo.com/dby/'
}, {
    'id'     : '6',
    'forumId': '52',
    'name'   : '崩坏：星穹铁道',
    'url'    : 'https://bbs.mihoyo.com/sr/'
}]


def random_text(num: int) -> str:
    return ''.join(random.sample(string.ascii_lowercase + string.digits, num))


class MihoyoBBSCoin:
    def __init__(self, cookies):
        self.headers = {
            'DS'                : old_version_get_ds_token(True),
            'cookie'            : cookies,
            'x-rpc-client_type' : '2',
            'x-rpc-app_version' : '2.7.0',
            'x-rpc-sys_version' : '6.0.1',
            'x-rpc-channel'     : 'mihoyo',
            'x-rpc-device_id'   : random_hex(32),
            'x-rpc-device_name' : random_text(random.randint(1, 10)),
            'x-rpc-device_model': 'Mi 10',
            'Referer'           : 'https://app.mihoyo.com',
            'Host'              : 'bbs-api.mihoyo.com',
            'User-Agent'        : 'okhttp/4.8.0'
        }
        self.Task_do = {
            'bbs_Sign'          : False,
            'bbs_Read_posts'    : False,
            'bbs_Read_posts_num': 3,
            'bbs_Like_posts'    : False,
            'bbs_Like_posts_num': 5,
            'bbs_Share'         : False
        }
        self.mihoyobbs_List_Use = []
        self.Today_getcoins = 0
        self.Today_have_getcoins = 0  # 这个变量以后可能会用上，先留着了
        self.Have_coins = 0

    async def task_run(self):
        await self.load_mihoyo_bbs_list_use()
        start = await self.get_tasks_list()
        self.postsList = await self.get_list()
        sign = await self.signing()
        read = await self.read_posts()
        like = await self.like_posts()
        share = await self.share_post()
        im = start + '\n' + sign + '\n' + read + '\n' + like + '\n' + share
        return im

    async def load_mihoyo_bbs_list_use(self):
        for i in [2, 5]:
            for k in mihoyobbs_List:
                if i == int(k['id']):
                    self.mihoyobbs_List_Use.append(k)

    # 获取任务列表，用来判断做了哪些任务
    async def get_tasks_list(self):
        # log.info('正在获取任务列表')
        async with AsyncClient() as client:
            req = await client.get(url=bbs_Taskslist, headers=self.headers)
        data = req.json()
        if 'err' in data['message'] or data['retcode'] == -100:
            return '你的Cookies已失效。'
            # log.error('获取任务列表失败，你的cookie可能已过期，请重新设置cookie。')
        else:
            self.Today_getcoins = data['data']['can_get_points']
            self.Today_have_getcoins = data['data']['already_received_points']
            self.Have_coins = data['data']['total_points']
            # 如果当日可获取米游币数量为0直接判断全部任务都完成了
            if self.Today_getcoins == 0:
                self.Task_do['bbs_Sign'] = True
                self.Task_do['bbs_Read_posts'] = True
                self.Task_do['bbs_Like_posts'] = True
                self.Task_do['bbs_Share'] = True
            else:
                # 如果第0个大于或等于62则直接判定任务没做
                if data['data']['states'][0]['mission_id'] >= 62:
                    # log.info(f'新的一天，今天可以获得{self.Today_getcoins}个米游币')
                    pass
                else:
                    # log.info(f'似乎还有任务没完成，今天还能获得{self.Today_getcoins}')
                    for i in data['data']['states']:
                        # 58是讨论区签到
                        if i['mission_id'] == 58:
                            if i['is_get_award']:
                                self.Task_do['bbs_Sign'] = True
                        # 59是看帖子
                        elif i['mission_id'] == 59:
                            if i['is_get_award']:
                                self.Task_do['bbs_Read_posts'] = True
                            else:
                                self.Task_do['bbs_Read_posts_num'] -= i['happened_times']
                        # 60是给帖子点赞
                        elif i['mission_id'] == 60:
                            if i['is_get_award']:
                                self.Task_do['bbs_Like_posts'] = True
                            else:
                                self.Task_do['bbs_Like_posts_num'] -= i['happened_times']
                        # 61是分享帖子
                        elif i['mission_id'] == 61:
                            if i['is_get_award']:
                                self.Task_do['bbs_Share'] = True
                                # 分享帖子，是最后一个任务，到这里了下面都是一次性任务，直接跳出循环
                                break
            return '开始执行~'

    # 获取要帖子列表
    async def get_list(self) -> list:
        temp_list = []
        print('正在获取帖子列表......')
        async with AsyncClient() as client:
            req = await client.get(url=bbs_Listurl.format(self.mihoyobbs_List_Use[0]['forumId']), headers=self.headers)
        data = req.json()
        for n in range(5):
            temp_list.append([data['data']['list'][n]['post']['post_id'], data['data']['list'][n]['post']['subject']])
        # log.info('已获取{}个帖子'.format(len(temp_list)))
        return temp_list

    # 进行签到操作
    async def signing(self):
        if self.Task_do['bbs_Sign']:
            return '讨论区任务已经完成过了~'
        else:
            for i in self.mihoyobbs_List_Use:
                async with AsyncClient() as client:
                    req = await client.post(url=bbs_Signurl.format(i['id']), data={}, headers=self.headers)
                data = req.json()
                if 'err' not in data['message']:
                    time.sleep(random.randint(2, 8))
                else:
                    return '你的Cookies已失效。'
            return '已完成签到任务~'

    # 看帖子
    async def read_posts(self):
        if self.Task_do['bbs_Read_posts']:
            return '看帖任务已经完成过了~'
        else:
            num_ok = 0
            for i in range(self.Task_do['bbs_Read_posts_num']):
                async with AsyncClient() as client:
                    req = await client.get(url=bbs_Detailurl.format(self.postsList[i][0]), headers=self.headers)
                data = req.json()
                if data['message'] == 'OK':
                    num_ok += 1
                time.sleep(random.randint(2, 8))
            return '已完成看帖任务~共计成功{}次~'.format(str(num_ok))

    # 点赞
    async def like_posts(self):
        if self.Task_do['bbs_Like_posts']:
            return '点赞任务已经完成过了~'
        else:
            num_ok = 0
            num_cancel = 0
            for i in range(self.Task_do['bbs_Like_posts_num']):
                async with AsyncClient() as client:
                    req = await client.post(url=bbs_Likeurl, headers=self.headers,
                                            json={'post_id': self.postsList[i][0], 'is_cancel': False})
                data = req.json()
                if data['message'] == 'OK':
                    num_ok += 1
                # 判断取消点赞是否打开
                if True:
                    time.sleep(random.randint(2, 8))
                    async with AsyncClient() as client:
                        req = await client.post(url=bbs_Likeurl, headers=self.headers,
                                                json={'post_id': self.postsList[i][0], 'is_cancel': True})
                    data = req.json()
                    if data['message'] == 'OK':
                        num_cancel += 1
                time.sleep(random.randint(2, 8))
            return '已完成点赞任务~共计点赞{}次，取消点赞{}次~'.format(str(num_ok), str(num_cancel))
            # 分享操作

    async def share_post(self):
        if self.Task_do['bbs_Share']:
            return '分享任务已经完成过了~'
        else:
            for _ in range(3):
                async with AsyncClient() as client:
                    req = await client.get(url=bbs_Shareurl.format(self.postsList[0][0]), headers=self.headers)
                data = req.json()
                if data['message'] == 'OK':
                    return '已完成分享任务~获得10米游币~'
                else:
                    time.sleep(random.randint(2, 8))
            time.sleep(random.randint(2, 8))



import requests, _thread, subprocess
import json,simplim,math,os
from bs4 import BeautifulSoup as bs
from simplim.premium import *
from simplim.api import pixiv
from requests.packages.urllib3.exceptions import InsecureRequestWarning
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)

# host = '210.140.131.203'
# pximg_host = '210.140.92.138'
try:
	host = socket.gethostbyname('pixiv.com')
except:
	host = '210.140.92.186'
try:
	pximg_host = socket.gethostbyname('pximg.net')
except:
	pximg_host = '210.140.92.143'
test_url='https://'+host+'/artworks/79025298?format=json'
headers = pixiv.headers

def getIllustInfo(pid):
	response = requests.get('https://'+host+'/artworks/'+str(pid), headers={'Host':'www.pixiv.net'},verify=False)
	if response.status_code != 200:
		return
	text = response.text
	soup = bs(text,'lxml')
	#open('h.html','w').write(text)
	content = soup.find('meta',id='meta-preload-data').get('content')
	dict_ = json.loads(content)
	dict_ = simdict(dict_)
	dict_.illust = simdict(dict_.illust[str(pid)])
	dict_.__dict__['manga_urls'] = []
	if dict_.illust.pageCount > 1:
		dict_.__dict__['manga']=True
		for page in range(0, dict_.illust.pageCount):
			s = dict()
			for url in dict_.illust.urls:
				s[url] = dict_.illust.urls[url].replace('p0','p'+str(page))
			dict_.__dict__['manga_urls'].append(s)
	else:
		dict_.__dict__['manga']=False
	return dict_

def getIllustInfoCompatible(pid):
	info = getIllustInfo(pid)
	processed = empty()
	if not info:
		return
	illust = info.illust
	for uid in info.user:
		user = info.user[uid]
	extra = illust.extraData.meta
	processed.info		= info
	processed.title		= illust.illustTitle
	processed.caption	= illust.illustComment
	processed.tags		= [ tag['tag'] for tag in illust.tags.tags ]
	processed.urls		= illust.urls
	processed.mini		= illust.urls.mini
	processed.thumb		= illust.urls.thumb
	processed.small		= illust.urls.small
	processed.regular	= illust.urls.regular
	processed.original	= illust.urls.original
	processed.width		= illust.width
	processed.height	= illust.height
	processed.createDate= illust.createDate
	processed.uploadDate= illust.uploadDate
	processed.age_limit	= illust.xRestrict
	processed.user		= user
	processed.pid		= illust.illustId
	processed.is_manga	= illust.pageCount != 1
	processed.manga		= []
	if processed.is_manga:
		pages	= range(1,illust.pageCount+1)
		for page in pages:
			processed.manga.append(processed.original.replace('p0','p'+str(page)))
	return simdict(processed.__dict__)

def getIllustUrls(pid):
	info = getIllustInfo(pid)
	if not info:
		return
	return info.illust.urls, info.manga_urls, info

def getIllust(pid, path='', use_curl=True, setting='original'):
	info = getIllustInfo(pid)
	if not info:
		return
	title= info.illust.title
	urls, manga_urls = info.illust.urls, info.manga_urls

	if manga_urls:
		print(manga_urls)
		if not path:
			path = title
		if not os.path.exists(path):
			os.mkdir(path)
		cnt=0
		for manga_url in manga_urls:
			ret = simplim.multiThreadDownload(url=manga_url[setting].replace('i.pximg.net',pximg_host),headers=pixiv.headers,path=os.path.join(path,str(cnt)+'.jpg'),verify=False)
			if not ret:
				# d=simplim.download(url=manga_url[setting],headers=pixiv.headers,need_content_length)
				# open(os.path.join(path,str(cnt)+'.jpg'),'wb').write(d.getvalue()).close()
				pixiv.getImg(url=manga_url[setting].replace('i.pximg.net',pximg_host),name=str(cnt),path=path,useCurl=use_curl)
			cnt+=1
	else:
		print(urls)
		if not path:
			path = './'
		ret = simplim.multiThreadDownload(url=urls[setting].replace('i.pximg.net',pximg_host),headers=pixiv.headers,path=os.path.join(path,title+'.jpg'),verify=False)
		if not ret:
			pixiv.getImg(url=urls[setting].replace('i.pximg.net',pximg_host), name=title,path=path,useCurl=use_curl)

def getIllustByUrl(url,path='',use_curl=False):
	# pixiv.getImg(url = url.replace('i.pximg.net',pximg_host),name=simplim.gettemp(),path=path,useCurl=use_curl)
	if use_curl:
		open(path,'wb').write(requests.get(url.replace('i.pximg.net',pximg_host), headers=headers, verify=False).content)
	else:
		simplim.multiThreadDownload(url=url.replace('i.pximg.net',pximg_host),headers=pixiv.headers,path=path,verify=False)

def getUserLikedWorks(pid,use_curl = False):
	headers = {
		'host':'www.pixiv.net',
		'cookie': myCookies
	}
	params = {
		'tag':'',
		'offset': '0',
		'limit': '100',
		'rest':'show',
		'lang':'zh'
	}
	# ?tag=&offset=0&limit=48&rest=show&lang=zh
	response = requests.get('https://'+host+'/ajax/user/'+str(pid)+'/illusts/bookmarks', params = params, headers = headers, verify=False)
	if response.status_code == 200:
		if response.json()['error']:
			return
	else:
		return
	print(response.json())
	sd = simdict(response.json())
	total = sd.body.total
	works = sd.body.works
	limit = int(params['limit'])
	pages = total/limit
	print('\n\n\n'+str(total)+'\n\n\n')
	if pages>1:
		pages = range(0,int(pages))
		offset = 0
		for page in pages:
			offset += limit
			params['offset']=str(offset)
			while 1:
				try:
					response = requests.get('https://'+host+'/ajax/user/'+str(pid)+'/illusts/bookmarks', params = params, headers = headers, verify=False,timeout=10)
					break
				except:
					print('agian')
			sd = simdict(response.json())
			works += sd.body.works
		final = simdict({'works': works})
		final.dump('liked_works.'+str(pid)+'.'+simplim.gettemp()+'.json')
		return final
		# params['limit']=str(total)
		# response = requests.get('https://'+host+'/ajax/user/'+str(pid)+'/illusts/bookmarks', params = params, headers = headers, verify=False)
		# sd = simdict(response.json())
		# sd.dump(str(pid)+'.'+simplim.gettemp()+'.json')
	sd.body.works.dump(str(pid)+'.'+simplim.gettemp()+'.json')
	return sd

def getMainPage(cookie=myCookies):
	headers = {
		'host':'www.pixiv.net',
		'referer':'https://www.pixiv.net',
		'cookie': cookie
	}
	url = 'https://'+host+'/ajax/top/illust?mode=all&lang=zh'
	response=requests.get(url, headers = headers, verify=False)
	print(response.status_code)
	if response.status_code == 200:
		if not response.json()['error']:
			return response.json()

def getUserFollowingUsers(pid):
	headers = {
		'host':'www.pixiv.net',
		'cookie': myCookies,
		'tag':'',
		'offset':'0',
		'limit':'24',
		'rest':'show',
		'lang':'zh'
	}
	# ?offset=0&limit=24&rest=show&tag=&lang=zh
	response=requests.get('https://'+host+'/ajax/user/'+str(pid)+'/following', headers = headers, verify=False)
	text = response.json()
	sd = simdict(text)
	total = sd.body.total
	users = sd.body.users
	limit = headers['limit']
	pages = total/limit
	#open('h.html','w').write(text)
	# content = soup.find('meta',id='meta-preload-data').get('content')
	# dict_ = json.loads(content)
	# dict_ = simdict(dict_)
	return soup

def getRank(mode='daily', page=1, content=''):
	'''
	mode
		daily
		weekly
		monthly
		rookie
		original
		male
		female
	content
		all
			all mode
		illust
			daily - rookie
		ugoira
			daily - weekly
		manga
			daily - rookie
	'''
	valid_mode = {
		'all': ['daily','weekly','monthly','rookie','original','male','female'],
		'illust': ['daily','weekly','monthly','rookie'],
		'ugoira': ['daily','weekly'],
		'manga': ['daily','weekly','monthly','rookie']
	}
	url = 'https://'+host+'/ranking.php'
	params = {
		'mode': mode,
		'p': str(page),
		'format': 'json'
	}
	if content:
		if mode not in valid_mode[content]:
			return
		else:
			params['content'] = content
	response = requests.get(url, params=params, headers={'Host':'www.pixiv.net'}, verify=False)
	if response.status_code == 200:
		if 'error' not in response.json():
			return response.json()

def getIllustRecommend(pid, limit=18):
	url = 'https://'+host+'/ajax/illust/'+str(pid)+'/recommend/init?limit='+str(limit)+'&lang=zh'
	response = requests.get(url, headers={'Host':'www.pixiv.net', 'cookie':myCookies, 'referer':'https://www.pixiv.net/artworks/'+str(pid)}, verify=False)
	if response.status_code == 200:
		if not response.json()['error']:
			return response.json()

def getMuiltiIllustsThumb(ids):
	url='https://'+host+'/ajax/illust/recommend/illusts'
	illust_ids = []
	for pid in ids:
		illust_ids.append('illust_ids%5B%5D='+str(pid))
	print('&'.join(illust_ids))
	url+='?'+'&'.join(illust_ids)+'&lang=zh'
	response = requests.get(url, headers={'Host':'www.pixiv.net'}, verify=False)
	if response.status_code == 200:
		if not response.json()['error']:
			return response.json()

def getLatestIllusts(page=1):
	url='https://'+host+'/bookmark_new_illust.php'
	params = {
		'p': page
	}
	response = requests.get(url,params=params, headers={'Host':'www.pixiv.net', 'cookie':myCookies}, verify=False)
	if response.status_code == 200:
		idname = 'js-mount-point-latest-following'
		text = response.text
		lxml = bs(text.replace('&quot;','\''),'lxml')
		data = lxml.find(id=idname).get('data-items')
		return json.loads(data.replace('\'','\"'))

def search(word,s_type='artworks',order='date_d',mode='all',p=1,s_mode='s_tag',type_='all',lang='zh',wlt=0,hlt=0):
	'''
		word: #url need to replace ' ' to %20
			mainWord -doninclude -anotherDoninclude (include OR anotherInclude)
			
			特殊词汇/标签
			オリジナル 原创
			10000users入り 10000以上收藏
			オリジナル10000users入り 原创10000以上收藏
			# 可以直接做成收藏筛选
			# 也可以做成搜索小提示
		s_type:
			artworks: 所有
			manga: 漫画
				type: manga
			illustrations: 插画，动图（继续用type定义）
				type: illust
					  ugoira
					  illust_and_ugoira

		order:
			date 正序
			date_d/任意 倒序
		mode:
			
		s_mode:
			s_tag
			s_tag_full
			s_tc

		ratio:
			0 正方形
			0.5 横
			-0.5 纵

		wlt: width 最小限制
		hlt: height 最小限制
		wgt: 最大
		hgt: 最大
		tool: 工具
			所有的制图工具
			SAI
			Photoshop
			CLIP STUDIO PAINT
			IllustStudio
			ComicStudio
			Pixia
			AzPainter4
			Painter
			Illustrator
			GIMP
			FireAlpaca
			網上描繪
			AzPainter
			CGillust
			描繪聊天室
			手畫博克
			MS_Paint
			PictBear
			openCanvas
			PaintShopPro
			EDGE
			drawr
			COMICWORKS
			AzDrawing
			SketchBookPro
			PhotoStudio
			Paintgraphic
			MediBang Paint
			NekoPaint
			Inkscape
			ArtRage
			AzDrawing4
			Fireworks
			ibisPaint
			AfterEffects
			mdiapp
			GraphicsGale
			Krita
			kokuban.in
			RETAS STUDIO
			emote
			4thPaint
			ComiLabo
			pixiv Sketch
			Pixelmator
			Procreate
			Expression
			PicturePublisher
			Processing
			Live2D
			dotpict
			Aseprite
			Poser
			Metasequoia
			Blender
			Shade
			3dsMax
			DAZ Studio
			ZBrush
			Comi Po!
			Maya
			Lightwave3D
			六角大王
			Vue
			SketchUp
			CINEMA4D
			XSI
			CARRARA
			Bryce
			STRATA
			Sculptris
			modo
			AnimationMaster
			VistaPro
			Sunny3D
			3D-Coat
			Paint 3D
			VRoid Studio
			筆芯筆
			鉛筆
			原子筆
			毫筆
			顏色鉛筆
			Copic麥克筆
			沾水筆
			透明水彩
			毛筆
			毛筆
			記號筆
			麥克筆
			水溶性彩色铅笔
			涂料
			丙烯顏料
			鋼筆
			粉彩
			噴筆
			顏色墨水
			蠟筆
			油彩
			COUPY-PENCIL
			顏彩
			蠟筆

		scd=2021-02-04 #开始时间 
		ecd=2021-03-05 #结束时间
	'''
	url ='https://'+host+'/ajax/search/'+s_type+'/'+word
	params = {
		'word':word,
		'order':order,
		'mode':mode,
		'p':p,
		's_mode':s_mode,
		'type':type_,
		'lang':lang
	}
	response = requests.get(url, params=params, headers={'Host':'www.pixiv.net'},verify=False)
	return response

# m = getUserLikedWorks(myId)
# m = getUserFollowingUsers(myId)

# z=requests.get('https://210.140.131.203/ajax/user/37265945/illusts/bookmarks?offset=0&limit=24&rest=show&tag=金的&lang=zh', headers={'host':'www.pixiv.net','cookie':myCookies}, verify=False)

# x=getUserLikedWorks(myId,0)
# print(len(x.works))



r"""@2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool using website <https://ascii2d.net/> & its search method <https://ascii2d.net/search/uri> to search & match anime pics which are mostly on Pixiv
"""

from bs4 import BeautifulSoup as bs
import simplim.api.translate.youdao as yd
from pathlib import Path
import requests as rq
import json
import re

host = 'https://ascii2d.net/'

url = 'http://tiebapic.baidu.com/forum/w%3D580/sign=5ab18faec02a60595210e1121835342d/f94dbade9c82d158c7013ad6970a19d8bd3e429d.jpg'

headers = {
	'origin': 'https://ascii2d.net',
	'referer': 'https://ascii2d.net/',
	'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/84.0.4147.125 Safari/537.36'
}

cookie = {
	'_session_id': '79565ed210ce8fcd11c72d20b8651189'
}

def getPage(url):
	data = {
		'authenticity_token': 'MY2WYZuFTuosuWtloftEIDs6NeA16P1F8FgNiUqI/kFDoetkQHzyxT8O2yC2I5cGcwTynFvewBuVB/JN7IQeTQ==',
		'uri': url,
		'utf-8': '✓',
		'search': ''
	}
	response = rq.post(url = 'https://ascii2d.net/search/uri', headers = headers, data = data, cookies = cookie)
	text = response.text
	url = response.url
	hash_self = url[re.search('color\/',url).end():]
	items = bs(text,'lxml').find_all(class_='row')[2].find_all(class_='item-box')
	final_results = []
	
	for item in items:
		thumbnail = host + item.find(class_='image-box').find('img').get('src')
		info = item.find(class_='info-box')
		hash_ = info.find(class_ = 'hash').text
		img_detail = info.find('small').text.split(' ')
		img_size = img_detail[0]
		start = re.search('x',img_size).start()
		width = img_size[:start]
		height = img_size[start+1:]
		img_type = img_detail[1]
		file_size = img_detail[2]
		detail = item.find(class_='detail-box')
		results = []
		if detail.text!='\n' and detail.text!='':
			site = detail.find('small').text
			if re.search('twitter',site):
				site = 'twitter'
			elif re.search('pixiv',site):
				site = 'pixiv'
			elif re.search('ニコニコ静画',site):
				site = 'ニコニコ静画'
			links = detail.find_all('a')
			for count in range(0, int(len(links)/2)):
				text1 = links[count*2].text
				text2 = links[count*2+1].text
				link1 = links[count*2].get('href')
				link2 = links[count*2+1].get('href')
				if site == 'twitter':
					results.append({
						'time': text1.split('.'),
						'work': link1,
						'user_name': text2,
						'user_link': link2,
						'user_id': link2[re.search('user_id=', link2).end():]
						})
				elif site == 'pixiv':
					results.append({
						'name': text1,
						'work': link1,
						'work-id': link1[re.search('artworks\/', link1).end():],
						'user_name': text2,
						'user_link': link2,
						'user_id': link2[re.search('users\/', link2).end():]
						})
				elif site == 'ニコニコ静画':
					results.append({
						'name': text1,
						'work': link1,
						'work-id': link1[re.search('seiga\/', link1).end():],
						'user_name': text2,
						'user_link': link2,
						'user_id': link2[re.search('illust\/', link2).end():]
						})
			if results:
				final_results.append({
					'site': site,
					'thumbnail': thumbnail,
					'img_detail':{
						'width': width,
						'height': height,
						'type': img_type,
						'size': file_size
						}, 
					'results':results,
					'hash': hash_
					})
	return {'hash':hash_self, 'results': final_results}







r"""@2020 HARDRELICE. All Rights Reserved.
 ╻ ╻ ┏━┓ ┏━┓ ┏━┓ ┏━┓ ┏━╸ ╻   ╻  ┏━╸ ┏━╸ TM
 ┣━┫ ┣━┫ ┣━┛ ┃ ┃ ┣━┛ ┣━╸ ┃   ┃  ┃   ┣━╸
 ╹ ╹ ╹ ╹ ╹ ╹ ┗━┛ ╹ ╹ ┗━╸ ┗━╸ ╹  ┗━╸ ┗━╸

A tool using <https://graph.baidu.com/> & <https://ai.baidu.com/aidemo> to upload, search, download pic on BaiduGraph & detect pic on BaiduAi
"""

import requests as rq
import json

def uploadImg(fileName):
	info = open(fileName,'rb').read()
	headers = {
		'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36',
		'referer':'http://tools.bugscaner.com/'
	}
	res = rq.Session()
	data = {
		'tn':'pc',
		'from':'pc',
		'image_source':'PC_UPLOAD_SEARCH_FILE',
		'range':'''{'page_from': 'searchResult'}''',
	}
	files = {'image':info}
	response = res.post('https://graph.baidu.com/upload',data=data,files=files,verify=False,headers=headers)
	returndict = response.json()
	sign = returndict['data']['sign']
	url  = returndict['data']['url']
	picurl = 'https://graph.baidu.com/resource/' + sign + '.jpg'
	print('[N]', 'search url', url)
	print('[N]', 'upload to', picurl)
	return url, picurl

def uploadImg_raw(info):
	headers = {
		'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36',
		'referer':'http://tools.bugscaner.com/'
	}
	res = rq.Session()
	data = {
		'tn':'pc',
		'from':'pc',
		'image_source':'PC_UPLOAD_SEARCH_FILE',
		'range':'''{'page_from': 'searchResult'}''',
	}
	files = {'image':info}
	response = res.post('https://graph.baidu.com/upload',data=data,files=files,verify=False,headers=headers)
	returndict = response.json()
	sign = returndict['data']['sign']
	url  = returndict['data']['url']
	picurl = 'https://graph.baidu.com/resource/' + sign + '.jpg'
	print('[N]','search url', url)
	print('[N]','upload to',picurl)
	return url, picurl

def searchImg(fileName):
	url, picurl = uploadImg(fileName)
	if returndict['msg'] == 'Success':
		print('[N] Success, starting to sparse resource')
		page = res.get(url, headers = headers, verify = False).text
		return page
	else:
		print('[E] Upload Failed')

def detectImg_ocr(url):
	host = 'https://ai.baidu.com/aidemo'
	data = {
		'image': '',
		'image_url': url,
		'type': 'commontext',
		'detect_direction': False
	}
	headers = {
		'user-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_13_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.122 Safari/537.36',
		'referer': 'https://ai.baidu.com/tech/ocr/general',
		'cookie':'PSTM=1588908167; BAIDUID=3D50572CC2497D155AAAD59A715ABFE1:FG=1; BDORZ=B490B5EBF6F3CD402E515D22BCDA1598; BIDUPSID=2737C11AF0424FE96F10064E74658ECF; MCITY=-158%3A218%3A; H_PS_PSSID=1450_31669_21080_31253_31596_31605_31271_31463_30823; BDUSS=0tSQzdXcmhVVjJZS0F4eXVMVXFzT3ROam1xVFNnVHlRSkwxdGt-S3l2LTA0TzVlRUFBQUFBJCQAAAAAAAAAAAEAAADnS~xbydnE6sjyzcEwMDEAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAALRTx160U8dec; delPer=0; PSINO=6; BCLID=7276149949773063458; BDSFRCVID=NtCOJeC62rw_J0ouvkQ5MRVUqfaPqNOTH6aImWQbBlut1nDH8-cBEG0P_x8g0K4bo7k0ogKKLgOTHULF_2uxOjjg8UtVJeC6EG0Ptf8g0f5; H_BDCLCKID_SF=tRAOoC8ytDvjDb7GbKTD-tFO5eT22-usL6bC2hcH0KLKJbOLhlb85JKHhJ5uQq5P-mTb0RT7tMb1MRjvWljO3-uhMt-eWbQUKmLJXq5TtUJoSDnTDMRhqqJXqqjyKMniBIv9-pnGBpQrh459XP68bTkA5bjZKxtq3mkjbPbDfn02eCKuD6K5j65XDGLs54JJ26c0LPI82P5Hfn5Y-Pvo5t3H-UnLq-J20mOZ0l8Ktt5voCOtjJo-hx-4XJ5pLRopJg3ZKl7mWIQHDPThefRhe5LSKtoAaUvZKJv4KKJxaMKWeIJo5t5BMh8shUJiBM7MBan7-lRIXKohJh7FM4tW3J0ZyxomtfQxtNRJ0DnjtnLhbC_wj50WjTjXeUQ0eJcybCnKW5rJabC3jb7cXU6qLT5Xj4LJbTDqb6cXKqjzLfQlqJcx0qjxjl0njxQyJUcI3ejIX4cqKKJqVnrj-xonDh8ebG7MJUntKHLOK-oO5hvv8b3O3M7lXMKmDloOW-TB5bbPLUQF5l8-sq0x0bOte-bQbG_EJjDeJJFHoDDQKROMjtQY5bu_MbL_5Uo05-7ybCPX3b7EfMj6Oq7_bf--DRLUWG-eQJQq22Jt2xoCQ4jqOpvRy-bxy5K_hUvd0t3vaHneBfQx2pnkH45HQT3mhpQbbN3i-4j82NRMWb3cWhRJ8UbS3fvPBTD02-nBat-OQ6npaJ5nJq5nhMJmb67JD-50eGLOt6FjJRC8VbOsa-5fK4bNh4bqhR3H-UnLqM6J02OZ0l8Kttob_pFxhno-Mpo0XJ5pLRohKGRD2q7mWIQHOJ650t5IhbL3DfTRBTkLK2r4KKJxX-PWeIJoj-5ojPnQhUJiBM7MBan7-lRIXKohJh7FM4tW3J0ZyxomtfQxtNRJ0DnjtnLhbRO4-TF5D5cBjfK; Hm_lvt_8b973192450250dd85b9011320b455ba=1590135564; Hm_lpvt_8b973192450250dd85b9011320b455ba=1590135919; __yjsv5_shitong=1.0_7_e61573df99b73a5e533b43fc32a9e83c649c_300_1590135919175_119.98.201.42_1f73e379'
	}
	try:
		response = rq.post(url=host, data=data, headers=headers).json()
	except:
		return -1
	# print('[R]',response)
	if response['msg']=='success':
		data = response['data']
		words = []
		string = ''
		for word in data['words_result']:
			words.append(word['words'])
			string += word['words']
		print('[N]','string',string)
		return words, string
	else: 
		return -1
























from math import *

def C(a, b):
	if a < b or not type(a) == type(0) or not type(b) == type(0) or b < 1:
		return 0
	return int(A(a, b) / A(b, b))

def A(a, b):
	if a < b or not type(a) == type(0) or not type(b) == type(0) or b < 1:
		return 0
	num = 1
	for a in range(a - b + 1, a + 1):
		num *= a
	return num

def D(n):
	if not type(n) == type(0):
		return 0
	else:
		if n <= 1:
			return 0
		if n == 2:
			return 1
		if n == 3:
			return 2
		else:
			return (n - 1) * (D(n - 1) + D(n - 2))

def mod(value1, value2):
	pt1 = value1//value2
	pt2 = value1 % value2
	return pt1,pt2

def any2decimal(base, value):
	length = len(value)-1
	decimal = 0
	for val in value:
		decimal += val * base ** length
		length -= 1
	return decimal

def decimal2any(base, decimal):
	value = []
	pt1 = -1
	while pt1!=0:
		pt1, pt2 = mod(decimal, base)
		value.insert(0,pt2)
		decimal = pt1
	return value

def base_convert(from_base, to_base, value):
	return decimal2any(to_base, any2decimal(from_base, value))

def string2ascii(string, bias = 0):
	value = []
	for char in string:
		value.append(ord(char)+bias)
	return value

def ascii2string(value, bias = 0):
	string = ''
	for val in value:
		string += chr(val+bias)
	return string

def string_from_base_list(value, base):
	form = base_list[base]
	string = ''
	for val in value:
		string+=form[val]
	return string

def value_from_base_list(string, base):
	form = base_list[base]
	value = []
	for char in string:
		value.append(form.index(char))
	return value

base_list = {
				58:'123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz',
				64:'ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxy0123456789+/',
				91:'ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxy0123456789!#$%&()*+,./:;<=>?@[]^_`{|}~"'
			}

def encode_base(string, base, use_ascii=False, bias = 33):
	if base in base_list and not use_ascii:
		return string_from_base_list(base_convert(256, base, string2ascii(string)), base)
	else:
		return ascii2string(base_convert(256, base, string2ascii(string)), bias = bias)

def decode_base(string, base, use_ascii=False, bias = 33):
	if base in base_list and not use_ascii:
		return ascii2string(base_convert(base, 256, value_from_base_list(string,base)))
	else:
		return ascii2string(base_convert(base, 256, string2ascii(string, bias =- bias)))

def b58encode(tmp:str) -> str:
	tmp = list(map(ord,tmp))
	temp = tmp[0]
	base58 = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
	for i in range(len(tmp)-1):
		temp = temp * 256 + tmp[i+1]
	tmp = []
	while True:
		tmp.insert(0,temp % 58)
		temp = temp // 58
		if temp == 0:
			break
	temp = ""
	for i in tmp:
		temp += base58[i]
	return temp

def b58decode(tmp:str) -> str:
	import binascii
	base58 = "123456789ABCDEFGHJKLMNPQRSTUVWXYZabcdefghijkmnopqrstuvwxyz"
	temp = []
	for i in tmp:
		temp.append(base58.index(i))
	tmp = temp[0]
	for i in range(len(temp)-1):
		tmp = tmp * 58 + temp[i+1]
	return binascii.unhexlify(hex(tmp)[2:].encode("utf-8")).decode("UTF-8")
